-
-- ========================
--  APEX Connect 2024
--  Gunther Pippèrr https://www.pipperr.de/dokuwiki/doku.php
--
-- =========================

set echo ON

-- ==============
-- Als SYS:
-- 
-- create directory PROFILER_DIR as '/opt/oracle/profiler';
-- 
-- grant all on directory PROFILER_DIR to GPI;
-- grant execute on dbms_hprof to GPI;
-- grant  SELECT_CATALOG_ROLE  to GPI;
--
-- =============
-- 
-- Als PL/SQL User:
-- 
-- execute DBMS_HPROF.CREATE_TABLES();
-- ======================================

accept step

-- ===== Start Demo ================

ALTER SESSION SET  PLSQL_OPTIMIZE_LEVEL=2;
ALTER SESSION SET  PLSQL_CODE_TYPE='INTERPRETED';
ALTER SESSION SET PLSQL_WARNINGS='ENABLE:ALL';
SET serveroutput ON


accept step

create or replace procedure profilerDemo
as
  v_loops  NUMBER := 100;
  v_start  NUMBER;
  v_return NUMBER;
  v_cpu_start number;

  FUNCTION add_numbers (p_1 IN NUMBER,
                        p_2 IN NUMBER)
    RETURN NUMBER AS
  BEGIN
    RETURN p_1 + p_2;
  END add_numbers;
  
  procedure lostTime
  is
     v_cnt pls_integer;
  begin
   select count(*) 
     into v_cnt
     from user_objects; 
  end;

BEGIN
  v_start     := DBMS_UTILITY.get_time;
  v_cpu_start := DBMS_UTILITY.GET_CPU_TIME; 

  FOR i IN 1 .. v_loops LOOP    
    v_return := add_numbers(1, i);
    lostTime;
  END LOOP;

   DBMS_OUTPUT.put_line('Elapsed Time: ' || (DBMS_UTILITY.get_time - v_start) || ' hsecs');
   DBMS_OUTPUT.put_line('Elapsed CPU Time: ' || (DBMS_UTILITY.GET_CPU_TIME - v_cpu_start) || ' hsecs');

END;
/

show error

accept step

begin
 dbms_hprof.start_profiling(  LOCATION => 'PROFILER_DIR' 
                                  ,  FILENAME => 'APEX_CONNECT_2024.trc');
end;
/

accept step

begin
profilerDemo;
end; 
/

accept step

begin
 dbms_hprof.stop_profiling;
end;
/

-- im OS plshprof /opt/oracle/profiler/Profile_run_15_03_2024.trc

-- =========== laden ==========

accept step

declare
   v_runid number;
Begin
   v_runid:=DBMS_HPROF.analyze('PROFILER_DIR'
                              ,'APEX_CONNECT_2024.trc');
end;
/

accept step

variable v_runid number

begin
select max(runid)
into :v_runid
from dbmshp_function_info;
end;
/

 select :v_runid from dual;
 
-- =========== auswerten ==========


with run_data as (
SELECT fi.symbolid,
               pci.parentsymid,
               RTRIM(fi.owner || '.' || fi.module || '.' || NULLIF(fi.function,fi.module), '.') AS name,
               NVL(pci.subtree_elapsed_time, fi.subtree_elapsed_time) AS subtree_elapsed_time,
               NVL(pci.function_elapsed_time, fi.function_elapsed_time) AS function_elapsed_time,
               NVL(pci.calls, fi.calls) AS calls
        FROM   dbmshp_function_info fi
               LEFT JOIN dbmshp_parent_child_info pci ON fi.runid = pci.runid AND fi.symbolid = pci.childsymid
        WHERE  fi.runid = :v_runid
        AND    fi.module != 'DBMS_HPROF'
)
SELECT RPAD(' ', (level-1)*2, ' ') || a.name AS name,
       a.subtree_elapsed_time,
       a.function_elapsed_time,
       a.calls
FROM   run_data  a
CONNECT BY a.parentsymid = PRIOR a.symbolid
START WITH a.parentsymid =1;

-- =====================


accept step



