-- ========================
--  APEX Connect 2024
--  Gunther Pippèrr https://www.pipperr.de/dokuwiki/doku.php?id=prog:plsql_bulk_collect
--
-- =========================

set echo ON

create or replace procedure bulck_load_test (p_testcase in pls_integer
                      , p_key in varchar2 )
as
  type cur_typ is ref cursor;

  type driver_id is table of varchar2 (9)  index by binary_integer;

  v_cur             cur_typ;
  v_driverids         driver_id;

  -- mit bind variablen
  p_sql            varchar2 (200) := 'update EMPLOYEES2 set SALARY = SALARY*0.19 where DEPARTMENT_ID = :key';
  p_getid_sql      varchar2 (200) := 'select EMPLOYEE_ID from EMPLOYEES2         where DEPARTMENT_ID like :key group by EMPLOYEE_ID order by 1';
  p_insert_sql     varchar2 (200) := 'update EMPLOYEES2 set SALARY = SALARY*0.19 where EMPLOYEE_ID   = :key';

  v_count          pls_integer := 0;
  v_errors         pls_integer := 0;
  v_message        varchar2 (100) := '';
  v_time_start     number := dbms_utility.get_time;                                      
  v_time_stop      number := 0;

  v_err_i        pls_integer := 0;
begin
  
    -- A) try in one statement: UPDATE als 1 SQL Statement
    
  if p_testcase = 1
  then
    execute immediate p_sql using p_key;

    v_count := sql%rowcount;
    commit;
    dbms_output.put_line ('-- Info :: SUCCESS rows ::' || v_count);
  
    -- B) try as loop: UPDATE als Loop with Ref Cursor 
  elsif p_testcase = 2
  then
  
    open v_cur for p_getid_sql using p_key;
        <<bulck_collect>>
    loop
    
      v_count := 0;
            v_errors := 0;

      fetch v_cur
            bulk collect into v_driverids
      limit 10000;

      --
      dbms_output.put_line ('-- Info ::read cound ids ::' || v_driverids.count);

      --
                                 
    -----------
    -- EXIT wenn Array leer!!
    
 	EXIT WHEN v_driverids.COUNT = 0;
    ------------
            

      -- Verarbeitung
      begin
        --forall i in v_driverids.first .. v_driverids.last save exceptions       -- sichere die Ausnahme, mache aber weiter!
          -- execute immediate p_alternativ_insert_sql using v_driverids (i);
                
                forall i in 1 .. v_driverids.COUNT save exceptions 
                    execute immediate p_insert_sql using v_driverids (i);
                    

        v_errors := v_errors + sql%bulk_exceptions.count;
        v_count := v_count + sql%rowcount;

        dbms_output.put_line ('-- Info ::SUCCESS rows::' || v_count);
        dbms_output.put_line ('-- Info ::Error rows::'   || v_errors);
                
         -- add a commit each 10000
         commit;
                
      exception
        when others
        then
          -- get the number of errors in the exception array
          v_errors := v_errors + sql%bulk_exceptions.count;
          v_message := sqlerrm;
          
          dbms_output.put_line ('-- Error::'      || v_message);
          dbms_output.put_line ('-- Error rows::' || v_errors);

          for i in 1 .. v_errors
          loop
            v_err_i := sql%bulk_exceptions (i).error_index;
            dbms_output.put_line ('ID=' || v_driverids (v_err_i) || ' error=' || sqlerrm (-1 * sql%bulk_exceptions (i).error_code));
          end loop;
      end;                                                                    

      
      
    end loop bulck_collect;                                                                     

    if v_cur%isopen = true
    then
      close v_cur;
    end if;
  -- C) all in one
  else
    open v_cur for p_getid_sql using p_key;

    fetch v_cur
    bulk collect into v_driverids;                                            -- alles auf einmal einlesen

        dbms_output.put_line ('-- Info :: read cound ids ::' || v_driverids.count);

    begin
      
      forall i in 1 .. v_driverids.count save exceptions         
        execute immediate p_insert_sql using v_driverids (i);

      v_errors := v_errors + sql%bulk_exceptions.count;
      v_count := v_count + sql%rowcount;

      dbms_output.put_line ('-- Info ::SUCCESS rows::' || v_count);
      dbms_output.put_line ('-- Info ::Error rows::'   || v_errors);

    exception
      when others
      then
        -- get the number of errors in the exception array
        v_errors := v_errors + sql%bulk_exceptions.count;
        v_message := sqlerrm;
        dbms_output.put_line ('-- Error::'      || v_message);
        dbms_output.put_line ('-- Error rows::' || v_errors);

        for i in 1 .. v_errors
        loop
          v_err_i := sql%bulk_exceptions (i).error_index;
          dbms_output.put_line (
            'ID=' || v_driverids (v_err_i) || ' error=' || sqlerrm (-1 * sql%bulk_exceptions (i).error_code));
        end loop;
    end;
        
    commit;
        
    close v_cur;
  end if;

  v_time_stop := dbms_utility.get_time;
  dbms_output.put_line (   '-- Info :: Variante: '
                 || p_testcase
                 || ' - Zeit:'
                 || to_char (round ( (v_time_stop - v_time_start) / 100, 2 ))
                 || ' sec');

end bulck_load_test;
/